/*

Il Gioco della Vita (Game of Life)  
+ una cella viva con 2 o 3 vicini vivi sopravvive, altrimenti muore
+ una cella morta con esattamente 3 vicini vivi rinasce

*/

// mondo: una matrice di celle
Cell[][] cells;     
int cellSize = 10;  
int numX, numY;     

void setup() { 
  frameRate(8);
  fullScreen();
  numX = floor(width/cellSize);
  numY = floor(height/cellSize);
  startup();
} 


void draw() {
  background(200);
            
  // calcolo il nuovo stato delle celle
  for (int x = 0; x < numX; x++) {
    for (int y = 0; y < numY; y++) {
     cells[x][y].compNextState();
    }
  }             
            
  // aggiorno lo stato e disegno le celle 
  for (int x = 0; x < numX; x++) {
    for (int y = 0; y < numY; y++) {
     cells[x][y].drawMe();
    }
  }
}

void startup() {
  // creo la griglia di celle
  cells = new Cell[numX][numY];  
  for (int x = 0; x < numX; x++) {
    for (int y = 0; y < numY; y++) {  
      cells[x][y] = new Cell(x, y);     
    }        
  }          
  // assegno i vicini ad ogni cella 
  for (int x = 0; x < numX; x++) {
    for (int y = 0; y < numY; y++) {  
      int above = y-1;    
      int below = y+1;    
      int left = x-1;      
      int right = x+1; 
      // lo spazio è circolare
      if (above < 0) above = numY-1;   
      if (below == numY) below = 0;  
      if (left < 0) left = numX-1;  
      if (right == numX) right = 0;  

      cells[x][y].addNeighbour(cells[left][above]);  
      cells[x][y].addNeighbour(cells[left][y]);    
      cells[x][y].addNeighbour(cells[left][below]);  
      cells[x][y].addNeighbour(cells[x][below]);  
      cells[x][y].addNeighbour(cells[right][below]);  
      cells[x][y].addNeighbour(cells[right][y]);  
      cells[x][y].addNeighbour(cells[right][above]);  
      cells[x][y].addNeighbour(cells[x][above]);    
    }
  }
}


void mousePressed() {
  startup();
}

class Cell {
  float x, y; // posizione
  boolean state;  // stato corrente
  boolean nextState;  // prossimo stato
  Cell[] neighbours; // vicini
  
  // costruttore di una cella
  Cell(float _x, float _y) {
    x = _x * cellSize;
    y = _y * cellSize;
    // scelgo lo stato casualmente
    if (random(2) > 1) {  
      nextState = true;
    } else {
      nextState = false; 
    }
    state = nextState;
    neighbours = new Cell[0];
  }
  
  // aggiungo un nuovo vicino alla cella corrente
  void addNeighbour(Cell cell) {
    neighbours = (Cell[]) append(neighbours, cell); 
  }
  
  // calcolo il nuovo stato
  void compNextState() { 
    // conto i vicini vivi
    int liveCount = 0;    
    for (int i=0; i < neighbours.length; i++) {
       if (neighbours[i].state == true) {  
         liveCount++;      
       }        
     }
    // se sono vivo e ho non troppi vicini vivi (2 o 3) rimango vivo, altrimenti muoio 
    if (state == true) {         
        if ((liveCount == 2) || (liveCount == 3)) {  
          nextState = true;          
        } else {            
          nextState = false;        
        } 
     // se sono morto e ho 3 vicini vivi, ritorno vivo         
     } else {            
        if (liveCount == 3) {        
          nextState = true;      
        } else {        
          nextState = false;    
        }        
     }       
  }

  // disegno la cella in base allo stato
  void drawMe() {
    state = nextState;
    noStroke();
    if (state == true) {
      // celle vive sono nere
      fill(0);
    } else {
      // celle morte sono bianche
      fill(255);
    }
    rect(x, y, cellSize, cellSize);
  }
  
}
